
#include <cstdio>
#include <cstring>
#include <exception>

#include "vm.hh"

PhysMem pm(4);

[[noreturn]] void
bad_fault(const char *p)
{
  std::fprintf(stderr, "page fault at %p\n", p);
  std::terminate();
}

int
main()
{
  VMRegion vmem(2 * page_size, bad_fault);
  PPage ppage = pm.page_alloc();

  VPage vp1 = vmem.get_base();
  VPage vp2 = vp1 + page_size;

  vmem.map(vp1, ppage, PROT_READ|PROT_WRITE);
  vmem.map(vp2, ppage, PROT_READ|PROT_WRITE);

  std::strcpy(vp1, "Copied this into page 1");
  printf("Page 1 says: %s\n", vp1);
  printf("Page 2 says: %s\n", vp2);

  std::strcpy(vp2, "Now I copied this into page 2");
  printf("Now page 1 says: %s\n", vp2);
  printf("Now page 2 says: %s\n", vp2);

  // You have to do this or program will crash
  vmem.unmap(vp1);
  vmem.unmap(vp2);
  pm.page_free(ppage);
}
