
#include <cstdio>
#include <random>

#include "vm.hh"

PhysMem pm(4);
std::default_random_engine dre;

struct FaultCatcher {
  VMRegion vmem;
  VPage last_page = nullptr;
  PPage ppage = pm.page_alloc();

  FaultCatcher() : vmem(0x1'0000, [this](char *va){ fault(va); }) {}
  ~FaultCatcher() {
    if (last_page)
      vmem.unmap(last_page);
    pm.page_free(ppage);
  }
  void fault(char *va);
};

void
FaultCatcher::fault(char *va)
{
  VPage vp = va - std::uintptr_t(va) % page_size;
  printf("Page fault at %p (page 0x%lx)\n", va,
	 (vp - vmem.get_base())/page_size);
  if (last_page)
    vmem.unmap(last_page);
  last_page = vp;
  vmem.map(vp, ppage, PROT_READ|PROT_WRITE);
}

int
main()
{
  FaultCatcher fc;
  char *const base = fc.vmem.get_base();
  std::uniform_int_distribution<int> rnd(0, fc.vmem.nbytes_-1);
  for (int i = 0; i < 20; ++i) {
    int idx = rnd(dre);
    printf("about to write at offset 0x%x (%p)\n", idx, &base[idx]);
    base[idx] = 'A';
  }
}
