
#include <iostream>
#include <string>

#include "ilist.hh"
#include "itree.hh"

template<typename Container>
void
print_all(Container &c)
{
  bool notfirst = false;
  for (const auto &e : c) {
    if (notfirst)
      std::cout << " -> ";
    else
      notfirst = true;
    std::cout << e.name;
  }
  std::cout << std::endl;
}

struct MyListEntry {
  std::string name;
  ilist_entry link;
  MyListEntry(std::string n) : name(std::move(n)) {}
};
using MyList = ilist<&MyListEntry::link>;

void
test_list()
{
  MyList l;
  l.push_back(new MyListEntry("A"));
  l.push_back(new MyListEntry("B"));
  l.push_back(new MyListEntry("C"));
  l.push_front(new MyListEntry("Z"));
  l.push_front(new MyListEntry("Y"));
  l.push_front(new MyListEntry("X"));
  print_all(l);
  l.delete_all();
}

struct MyTreeEntry {
  const std::string name;
  itree_entry tlink;
  MyTreeEntry(std::string n) : name(std::move(n)) {}
};
using MyTree = itree<&MyTreeEntry::name, &MyTreeEntry::tlink>;

void
test_tree()
{
  MyTree t;
  t.insert(new MyTreeEntry("A"));
  t.insert(new MyTreeEntry("Z"));
  t.insert(new MyTreeEntry("Y"));
  t.insert(new MyTreeEntry("B"));
  t.insert(new MyTreeEntry("C"));
  t.insert(new MyTreeEntry("X"));
  print_all(t);
  t.delete_all();
}

struct LruEntry {
  const std::string name;
  itree_entry name_link;
  ilist_entry lru_link;
  LruEntry(std::string n) : name(std::move(n)) {}
};

struct Lru {
  itree<&LruEntry::name, &LruEntry::name_link> index;
  ilist<&LruEntry::lru_link> lru;

  ~Lru() { lru.delete_all(); }

  void touch(std::string name) {
    if (LruEntry *e = index[name]) {
      lru.remove(e);
      lru.push_back(e);
    }
    else {
      e = new LruEntry(name);
      index.insert(e);
      lru.push_back(e);
    }
  }
};

void
test_lru()
{
  Lru l;
  l.touch("A");
  l.touch("B");
  l.touch("C");
  l.touch("D");
  print_all(l.lru);
  l.touch("C");
  l.touch("B");
  l.touch("A");
  print_all(l.lru);
}

int
main()
{
  test_list();
  //test_tree();
  //test_lru();
}
